@@ -590,9 +590,7 @@ class MambaModel(MambaPreTrainedModel):
         return_dict = return_dict if return_dict is not None else self.config.use_return_dict
 
         if (input_ids is None) ^ (inputs_embeds is not None):  # ^ is python for xor
-            raise ValueError(
-                "You cannot specify both input_ids and inputs_embeds at the same time, and must specify either one"
-            )
+            raise ValueError("You must specify exactly one of input_ids or inputs_embeds")
 
         if inputs_embeds is None:
             inputs_embeds = self.embeddings(input_ids)
@@ -709,6 +707,8 @@ class MambaForCausalLM(MambaPreTrainedModel, GenerationMixin):
         attention_mask: Optional[torch.LongTensor] = None,
         **kwargs,
     ):
+        # Overwitten -- uses `cache_params` as opposed to `past_key_values`
+
         if use_cache:
             # `cache_position` should have been initialized in `generate`
             if cache_position is None:
